from rapidata.rapidata_client.validation.rapids.rapids import Rapid
from rapidata.service.openapi_service import OpenAPIService
from requests.adapters import HTTPAdapter, Retry
import requests
from rapidata.api_client.models.update_dimensions_model import UpdateDimensionsModel

class RapidataValidationSet:
    """A class for interacting with a Rapidata validation set.

    Represents a set of all the validation tasks that can be added to an order.

    When added to an order, the tasks will be selected randomly from the set.

    Attributes:
        id (str): The ID of the validation set.
        name (str): The name of the validation set.
    """

    def __init__(self, validation_set_id, name: str, openapi_service: OpenAPIService):
        self.id = validation_set_id
        self.name = name
        self.__openapi_service = openapi_service
        self.__session = self._get_session()
    
    def add_rapid(self, rapid: Rapid):
        """Add a Rapid to the validation set.

        Args:
            rapid (Rapid): The Rapid to add to the validation set.
        """
        rapid._add_to_validation_set(self.id, self.__openapi_service, self.__session)
        return self

    def update_dimensions(self, dimensions: list[str]):
        """Update the dimensions of the validation set.

        Args:
            dimensions (list[str]): The new dimensions of the validation set.
        """
        self.__openapi_service.validation_api.validation_validation_set_id_dimensions_patch(self.id, UpdateDimensionsModel(dimensions=dimensions))
        return self

    def _get_session(self, max_retries: int = 5, max_workers: int = 10) -> requests.Session:   
        """Get a requests session with retry logic.


        Args:
            max_retries (int): The maximum number of retries.
            max_workers (int): The maximum number of workers.

        Returns:
            requests.Session: A requests session with retry logic.
        """
        session = requests.Session()
        retries = Retry(
            total=max_retries,
            backoff_factor=1,
            status_forcelist=[500, 502, 503, 504],
            allowed_methods=["GET"],
            respect_retry_after_header=True
        )

        adapter = HTTPAdapter(
            pool_connections=max_workers * 2,
            pool_maxsize=max_workers * 4,
            max_retries=retries
        )
        session.mount('http://', adapter)
        session.mount('https://', adapter)

        return session

    def __str__(self):
        return f"name: '{self.name}' id: {self.id}"

    def __repr__(self):
        return f"name: '{self.name}' id: {self.id}"



# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import io
import json
import re
from typing import Dict, Optional

import httpx
from authlib.integrations.httpx_client import OAuth2Client
from httpx import Timeout

from rapidata.api_client.exceptions import ApiException, ApiValueError


class RESTResponse(io.IOBase):

    def __init__(self, resp: httpx.Response) -> None:
        self.response = resp
        self.status = resp.status_code
        self.reason = resp.reason_phrase
        self.data = None

    def read(self):
        if self.data is None:
            self.data = self.response.content
        return self.data

    def getheaders(self) -> Dict[str, str]:
        """Returns a dictionary of the response headers."""
        return dict(self.response.headers)

    def getheader(self, name, default=None) -> Optional[str]:
        """Returns a given response header."""
        return self.response.headers.get(name, default)


RESTResponseType = RESTResponse


class RESTClientObject:

    def __init__(self, configuration) -> None:
        self.configuration = configuration

        self.session: Optional[OAuth2Client] = None

    def setup_oauth_client_credentials(
        self, client_id: str, client_secret: str, token_endpoint: str, scope: str
    ):
        client_args = self._get_session_defaults()
        self.session = OAuth2Client(
            client_id=client_id,
            client_secret=client_secret,
            token_endpoint=token_endpoint,
            scope=scope,
            **client_args,
        )

        self.session.fetch_token()

    def setup_oauth_with_token(
        self,
        client_id: str | None,
        client_secret: str | None,
        token: dict,
        token_endpoint: str,
        leeway: int = 60,
    ):
        client_args = self._get_session_defaults()
        self.session = OAuth2Client(
            token=token,
            token_endpoint=token_endpoint,
            client_id=client_id,
            client_secret=client_secret,
            leeway=leeway,
            **client_args,
        )

    def request(
        self,
        method,
        url,
        headers=None,
        body=None,
        post_params=None,
        _request_timeout=None,
    ):
        """Perform requests.

        :param method: http request method
        :param url: http request url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters,
                            `application/x-www-form-urlencoded`
                            and `multipart/form-data`
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        """
        method = method.upper()
        assert method in ["GET", "HEAD", "DELETE", "POST", "PUT", "PATCH", "OPTIONS"]

        if post_params and body:
            raise ApiValueError(
                "body parameter cannot be used with post_params parameter."
            )

        post_params = post_params or {}
        headers = headers or {}

        if not self.session:
            raise ApiValueError(
                "OAuth2 session is not initialized. Please initialize it before making requests."
            )

        session = self.session

        timeout = None
        if _request_timeout:
            if isinstance(_request_timeout, (int, float)):
                timeout = Timeout(timeout=_request_timeout)
            elif isinstance(_request_timeout, tuple) and len(_request_timeout) == 2:
                connect_timeout, read_timeout = _request_timeout
                timeout = Timeout(timeout=connect_timeout, read=read_timeout)

        try:
            if method in ["POST", "PUT", "PATCH", "OPTIONS", "DELETE"]:
                content_type = headers.get("Content-Type")

                if not content_type or re.search("json", content_type, re.IGNORECASE):
                    request_body = None
                    if body is not None:
                        request_body = json.dumps(body)
                    r = session.request(
                        method,
                        url,
                        content=request_body,
                        timeout=timeout,
                        headers=headers,
                    )

                elif content_type == "application/x-www-form-urlencoded":
                    r = session.request(
                        method, url, data=post_params, timeout=timeout, headers=headers
                    )

                elif content_type == "multipart/form-data":
                    del headers["Content-Type"]
                    files = []
                    data = {}

                    for key, value in post_params:
                        if isinstance(value, tuple) and len(value) >= 2:
                            # This is a file tuple (filename, file_data, [content_type])
                            filename, file_data = value[0], value[1]
                            content_type = value[2] if len(value) > 2 else None
                            files.append((key, (filename, file_data, content_type)))
                        elif isinstance(value, dict):
                            # JSON-serialize dictionary values
                            if key in data:
                                # If we already have this key, handle as needed
                                # (convert to list or append to existing list)
                                if not isinstance(data[key], list):
                                    data[key] = [data[key]]
                                data[key].append(json.dumps(value))
                            else:
                                data[key] = json.dumps(value)
                        else:
                            # Regular form data
                            if key in data:
                                if not isinstance(data[key], list):
                                    data[key] = [data[key]]
                                data[key].append(value)
                            else:
                                data[key] = value
                    r = session.request(
                        method,
                        url,
                        files=files,
                        data=data,
                        timeout=timeout,
                        headers=headers,
                    )

                elif isinstance(body, str) or isinstance(body, bytes):
                    r = session.request(
                        method, url, content=body, timeout=timeout, headers=headers
                    )

                elif headers["Content-Type"].startswith("text/") and isinstance(
                    body, bool
                ):
                    request_body = "true" if body else "false"
                    r = session.request(
                        method,
                        url,
                        content=request_body,
                        timeout=timeout,
                        headers=headers,
                    )

                else:
                    msg = """Cannot prepare a request message for provided arguments.
                             Please check that your arguments match declared content type."""
                    raise ApiException(status=0, reason=msg)

            else:
                r = session.request(
                    method, url, timeout=timeout, headers=headers
                )

        except httpx.HTTPError as e:
            msg = "\n".join([type(e).__name__, str(e)])
            raise ApiException(status=0, reason=msg)

        return RESTResponse(r)

    def _get_session_defaults(self):
        client_kwargs = {
            "verify": (
                self.configuration.ssl_ca_cert
                if self.configuration.ssl_ca_cert
                else self.configuration.verify_ssl
            )
        }

        if self.configuration.proxy:
            client_kwargs["proxy"] = self.configuration.proxy

            existing_headers = client_kwargs.pop("headers")
            if self.configuration.proxy_headers:
                for key, value in self.configuration.proxy_headers.items():
                    existing_headers[key] = value
            client_kwargs["headers"] = existing_headers

        if self.configuration.retries is not None:
            transport = httpx.HTTPTransport(retries=self.configuration.retries)
            client_kwargs["transport"] = transport

        return client_kwargs
